@@ -68,6 +68,8 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   public final int getPC() { return PC; }
 
+  public final int getSP() { return SP; }
+
   public final ConstantPool getConstants ()
   {
     return getMethod().classfile.constants;
@@ -247,11 +249,22 @@ public class CodeAttr extends Attribute implements AttrContainer
     reserve(1);
     Type type1 = popType();
     Type type2 = popType();
+
     if (type1.size > 4 || type2.size > 4)
-      throw new Error ("emitSwap:  not allowed for long or double");
-    pushType(type1);
-    put1(95);  // swap
-    pushType(type2);
+      {
+	// There is no swap instruction in the JVM for this case.
+	// Fall back to a more convoluted way.
+	pushType(type2);
+	pushType(type1);
+	emitDupX();
+	emitPop(1);
+      }
+    else
+      {
+	pushType(type1);
+	put1(95);  // swap
+	pushType(type2);
+      }
   }
 
   /** Emit code to duplicate the top element of the stack. */
@@ -393,10 +406,7 @@ public class CodeAttr extends Attribute implements AttrContainer
     Scope scope = locals.current_scope;
     locals.current_scope = scope.parent;
     scope.end_pc = PC;  readPC = PC;
-    for (Variable var = scope.vars; var != null; var = var.next) {
-      if (var.isSimple () && ! var.dead ())
-	var.freeLocal(this);
-    }
+    scope.freeLocals(this);
     return scope;
   }
 
@@ -677,8 +687,8 @@ public class CodeAttr extends Attribute implements AttrContainer
 	      emitInvokeVirtual(Type.string_type.getDeclaredMethod("intern", 0));
 	    return;
 	  }
+	pushType(Type.string_type);
       }
-    pushType(Type.string_type);
   }
 
   public void emitPushNull ()
@@ -971,13 +981,30 @@ public class CodeAttr extends Attribute implements AttrContainer
   public final void emitMul () { emitBinop (104); }
   public final void emitDiv () { emitBinop (108); }
   public final void emitRem () { emitBinop (112); }
-  public final void emitShl () { emitBinop (120); }
-  public final void emitShr () { emitBinop (122); }
-  public final void emitUshr() { emitBinop (124); }
   public final void emitAnd () { emitBinop (126); }
   public final void emitIOr () { emitBinop (128); }
   public final void emitXOr () { emitBinop (130); }
 
+  public final void emitShl () { emitShift (120); }
+  public final void emitShr () { emitShift (122); }
+  public final void emitUshr() { emitShift (124); }
+
+  private void emitShift (int base_code)
+  {
+    Type type2 = popType().promote();
+    Type type1_raw = popType();
+    Type type1 = type1_raw.promote();
+
+    if (type1 != Type.int_type && type1 != Type.long_type)
+      throw new Error ("the value shifted must be an int or a long");
+
+    if (type2 != Type.int_type)
+      throw new Error ("the amount of shift must be an int");
+
+    emitTypedOp(base_code, type1);
+    pushType(type1_raw);
+  }
+
   public final void emitNot(Type type)
   {
     emitPushConstant(1, type);
@@ -1011,7 +1038,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   }
 
   /**
-   * Comple code to push the contents of a local variable onto the statck.
+   * Compile code to push the contents of a local variable onto the statck.
    * @param var The variable whose contents we want to push.
    */
   public final void emitLoad (Variable var)
@@ -1169,6 +1196,8 @@ public class CodeAttr extends Attribute implements AttrContainer
       opcode = 184;   // invokestatic
     else if (method.classfile.isInterface())
       opcode = 185;   // invokeinterface
+    else if ("<init>".equals(method.getName()))
+      opcode = 183;   // invokespecial
     else
       opcode = 182;   // invokevirtual
     emitInvokeMethod(method, opcode);
@@ -1247,53 +1276,6 @@ public class CodeAttr extends Attribute implements AttrContainer
     emitGoto(label, 168);
   }
 
-  public final void emitGotoIfEq (Label label, boolean invert)
-  {
-    Type type2 = popType().promote();
-    Type type1 = popType().promote();
-    reserve(4);
-    int opcode;
-    char sig1 = type1.getSignature().charAt(0);
-    char sig2 = type2.getSignature().charAt(0);
-    if (sig1 == 'I' && sig2 == 'I')
-      opcode = 159;  // if_icmpeq (inverted: if_icmpne)
-    else if (sig1 == 'J' && sig2 == 'J')
-      {
-	put1(148);   // lcmp
-	opcode = 153;  // ifeq (inverted: ifne)
-      }
-    else if (sig1 == 'F' && sig2 == 'F')
-      {
-	put1(149);   // fcmpl
-	opcode = 153;  // ifeq (inverted: ifne)
-      }
-    else if (sig1 == 'D' && sig2 == 'D')
-      {
-	put1(151);   // dcmpl
-	opcode = 153;  // ifeq (inverted: ifne)
-      }
-    else if ((sig1 == 'L' || sig1 == '[')
-	     && (sig2 == 'L' || sig2 == '['))
-      opcode = 165;  // if_acmpeq (inverted: if_acmpne)
-    else
-      throw new Error ("non-matching types to emitGotoIfEq");
-    if (invert)
-      opcode++;
-    emitTransfer (label, opcode);
-  }
-
-  /** Compile a conditional transfer if 2 top stack elements are equal. */
-  public final void emitGotoIfEq (Label label)
-  {
-    emitGotoIfEq(label, false);
-  }
-
-  /** Compile conditional transfer if 2 top stack elements are not equal. */
-  public final void emitGotoIfNE (Label label)
-  {
-    emitGotoIfEq(label, true);
-  }
-
   public final void emitGotoIfCompare1 (Label label, int opcode)
   {
     popType();
@@ -1316,8 +1298,8 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   public final void emitGotoIfCompare2 (Label label, int logop)
   { 
-    if( logop < 155 || logop > 158 )
-      throw new Error ("emitGotoIfCompare2: logop must be one of iflt, ifgt, ifle, ifge");
+    if( logop < 153 || logop > 158 )
+      throw new Error ("emitGotoIfCompare2: logop must be one of ifeq...ifle");
     
     Type type2 = popType().promote();
     Type type1 = popType().promote();
@@ -1335,13 +1317,35 @@ public class CodeAttr extends Attribute implements AttrContainer
       put1(cmpg ? 149 : 150);   // fcmpl/fcmpg
     else if (sig1 == 'D' && sig2 == 'D')
       put1(cmpg ? 151 : 152);   // dcmpl/dcmpg
+    else if ((sig1 == 'L' || sig1 == '[')
+	     && (sig2 == 'L' || sig2 == '[')
+	     && logop <= 154)
+      logop += 12; // ifeq->if_acmpeq, ifne->if_acmpne
     else
-      throw new Error ("non-matching types to emitGotoIfCompare2");
+      throw new Error ("invalid types to emitGotoIfCompare2");
 
     emitTransfer (label, logop);
   }
 
   // binary comparisons
+  /** @deprecated */
+  public final void emitGotoIfEq (Label label, boolean invert)
+  {
+    emitGotoIfCompare2(label, invert ? 154 : 153);
+  }
+
+  /** Compile a conditional transfer if 2 top stack elements are equal. */
+  public final void emitGotoIfEq (Label label)
+  {
+    emitGotoIfCompare2(label, 153);
+  }
+
+  /** Compile conditional transfer if 2 top stack elements are not equal. */
+  public final void emitGotoIfNE (Label label)
+  {
+    emitGotoIfCompare2(label, 154);
+  }
+
   public final void emitGotoIfLt(Label label)
   { emitGotoIfCompare2(label, 155); }
   public final void emitGotoIfGe(Label label)
@@ -1659,6 +1663,13 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   public void emitCheckcast (Type type)
   {
+    if (type instanceof ClassType)
+      {
+	Type top = topType();
+	if (top instanceof ClassType
+	    && ((ClassType) top).isSubclass((ClassType) type))
+	  return;
+      }
     emitCheckcast(type, 192);
     pushType(type);
   }
@@ -1691,6 +1702,43 @@ public class CodeAttr extends Attribute implements AttrContainer
     put1 (195);  // monitorexit
   }
 
+  /** Call pending finalizer functions.
+   * @param limit Only call finalizers more recent than this.
+   */
+  public void doPendingFinalizers (TryState limit)
+  {
+    TryState stack = try_stack;
+
+    /* If a value is returned, it must be saved to a local variable,
+       to prevent a verification error because of inconsistent stack sizes.
+    */
+    boolean saveResult = ! getMethod().getReturnType().isVoid();
+    Variable result = null;
+
+    while (stack != limit)
+      {
+	if (stack.finally_subr != null         // there is a finally block
+	    && stack.finally_ret_addr == null) // 'return' is not inside it
+	  {
+	    if (saveResult && result == null)
+	      {
+		result = addLocal(topType());
+		emitStore(result);
+	      }
+	    emitJsr(stack.finally_subr);
+	  }
+
+	stack = stack.previous;
+      }
+
+    if (result != null)
+      emitLoad(result);
+    // We'd like to do freeLocal on the result Variable, but then we risk
+    // it being re-used in a finalizer, which would trash its value.  We
+    // don't have any convenient way to to do that (the pending Scope
+    // mechanism is over-kill), we for now we just leak the Variable.
+  }
+
   /**
    * Compile a method return.
    * If inside a 'catch' clause, first call 'finally' clauses.
@@ -1699,15 +1747,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    */
   public final void emitReturn ()
   {
-    TryState stack = try_stack;
-    while (stack != null)
-      {
-	if (stack.finally_subr != null         // there is a finally block
-	    && stack.finally_ret_addr == null) // 'return' is not inside it
-	  emitJsr(stack.finally_subr);
-	stack = stack.previous;
-      }
-
+    doPendingFinalizers(null);
     if (getMethod().getReturnType().size == 0)
       {
 	reserve(1);
@@ -1890,6 +1930,11 @@ public class CodeAttr extends Attribute implements AttrContainer
     try_stack = try_stack.previous;
   }
 
+  public final TryState getCurrentTry ()
+  {
+    return try_stack;
+  }
+
   public final boolean isInTry()
   {
     // This also return true if we're in  a catch clause, but that is
@@ -1897,7 +1942,7 @@ public class CodeAttr extends Attribute implements AttrContainer
     return try_stack != null;
   }
 
-  /** Compile a tail-call to position 0 of the current procewure.
+  /** Compile a tail-call to position 0 of the current procedure.
    * @param pop_args if true, copy argument registers (except this) from stack.
    * @param scope Scope whose start we jump back to. */
   public void emitTailCall (boolean pop_args, Scope scope)
