@@ -10,6 +10,13 @@ public class PrimType extends Type {
     Type.registerTypeForClass(reflectClass, this);
   }
 
+  protected PrimType(PrimType type)
+  {
+    super(type.this_name, type.signature);
+    size = type.size;
+    reflectClass = type.reflectClass;
+  }
+
   public Object coerceFromObject (Object obj)
   {
     if (obj.getClass() == reflectClass)
@@ -39,11 +46,47 @@ public class PrimType extends Type {
 
   /** Coerce value to a boolean.
    * Only defined if getSignature() is "Z". */
-  public boolean booleanValue (Object value)
+  public static boolean booleanValue (Object value)
   {
     return ! (value instanceof Boolean) || ((Boolean) value).booleanValue();
   }
 
+  public void emitCoerceToObject (CodeAttr code)
+  {
+    char sig1 = getSignature().charAt(0);
+    ClassType clas;
+    Method method;
+    String cname = null;
+    Type[] args;
+    switch (sig1)
+      {
+      case 'Z':
+	clas = ClassType.make("java.lang.Boolean");
+	code.emitIfIntNotZero();
+	code.emitGetStatic(clas.getDeclaredField("TRUE"));
+	code.emitElse();
+	code.emitGetStatic(clas.getDeclaredField("FALSE"));
+	code.emitFi();
+	break;
+      case 'C':  cname = "java.lang.Character"; break;
+      case 'B':  cname = "java.lang.Byte";      break;
+      case 'S':  cname = "java.lang.Short";     break;
+      case 'I':  cname = "java.lang.Integer";   break;
+      case 'J':  cname = "java.lang.Long";      break;
+      case 'F':  cname = "java.lang.Float";     break;
+      case 'D':  cname = "java.lang.Double";    break;
+      default:   cname = null;
+      }
+    clas = ClassType.make(cname);
+    args = new Type[1];
+    args[0] = this;
+    method = clas.getDeclaredMethod("<init>", args);
+    code.emitNew(clas);
+    code.emitDupX();
+    code.emitSwap();
+    code.emitInvokeSpecial(method);
+  }
+
   public void emitCoerceFromObject (CodeAttr code)
   {
     char sig1 = (signature == null || signature.length() != 1) ? ' '
@@ -92,10 +135,10 @@ public class PrimType extends Type {
     if (sig1 == 'Z' || sig2 == 'Z')
       return -3;
 
-    // On the other hand, in Java this is not correct,
-    // but it is correct for Scheme.  FIXME.
-    if (sig1 == 'C' || sig2 == 'C')
-      return -3;
+    if (sig1 == 'C')
+      return type2.size > 2 ? -1 : -3;
+    if (sig2 == 'C')
+      return type1.size > 2 ? 1 : -3;
 
     if (sig1 == 'D')
       return 1;
@@ -146,6 +189,9 @@ public class PrimType extends Type {
           return -1;
         break;
       }
+    if (otherName.equals("java.lang.Object")
+	|| other == tostring_type)
+      return -1;
     return -2;
   }
 }
