@@ -13,7 +13,7 @@ import gnu.kawa.lispexpr.LispInterpreter;
 public abstract class Lisp2 extends LispInterpreter
 {
   public static final LList FALSE = LList.Empty;
-  public static Binding TRUE;
+  public static Symbol TRUE;
   public static final Expression nilExpr = new QuoteExp(FALSE);
 
   public boolean isTrue(Object value)
@@ -44,6 +44,16 @@ public abstract class Lisp2 extends LispInterpreter
     return true;
   }
 
+  public int getNamespaceOf(Declaration decl)
+  {
+    // This is a kludge because the hygiene renameing in SyntaxRules
+    // (which is used for some macros that Lisp uses) doesn't distinguish
+    // function and variable position.
+    if (decl.isAlias())
+      return FUNCTION_NAMESPACE+VALUE_NAMESPACE;
+    return decl.isProcedureDecl() ? FUNCTION_NAMESPACE : VALUE_NAMESPACE;
+  }
+
   public void defineFunction(String name, Object proc)
   {
     Environment.defineFunction(environ, name, proc);
@@ -54,7 +64,7 @@ public abstract class Lisp2 extends LispInterpreter
   {
     if (name == "nil")
       return FALSE;
-    return Environment.getCurrentBinding(name);
+    return Environment.getCurrentSymbol(name);
     //return name;
   }
 
@@ -65,7 +75,7 @@ public abstract class Lisp2 extends LispInterpreter
   }
 
   /** Get a string for a given symbol. */
-  public static Object getString (Binding symbol)
+  public static Object getString (Symbol symbol)
   {
     return getString(symbol.getName());
   }
