@@ -1,12 +1,18 @@
-// Copyright (c) 2002  Per M.A. Bothner.
+// Copyright (c) 2002, 2003  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
 import gnu.mapping.*;
 import gnu.bytecode.CodeAttr;
+import gnu.bytecode.ClassType;
+//import gnu.bytecode.Method;
 import gnu.bytecode.Type;
 import gnu.lists.*;
 import gnu.text.Lexer;
+import gnu.text.SourceMessages;
+import gnu.kawa.reflect.*;
+import java.io.*;
+import java.lang.reflect.InvocationTargetException;
 
 /**
  * Contains various language-dependent methods.
@@ -48,6 +54,7 @@ public abstract class Interpreter
     { "brl", ".brl", "gnu.kawa.brl.BRL" },
     { "emacs", "elisp", "emacs-lisp", ".el", "gnu.jemacs.lang.ELisp" },
     { "xquery", ".xql", "gnu.xquery.lang.XQuery" },
+    { "xslt", "xsl", ".xsl", "gnu.kawa.xslt.XSLT" },
     { "commonlisp", "common-lisp", "clisp", "lisp",
       ".lisp", ".lsp", ".cl",
       "gnu.commonlisp.lang.CommonLisp" }
@@ -148,9 +155,14 @@ public abstract class Interpreter
     catch (Exception ex)
       {
 	langName = langClass.getName();
-	ex.printStackTrace();
+	Throwable th;
+	if (ex instanceof InvocationTargetException)
+	  th = ((InvocationTargetException) ex).getTargetException();
+	else
+	  th = ex;
+	// th.printStackTrace();
 	throw new WrappedException("getInstance for '" + langName + "' failed",
-				   ex);
+				   th);
       }
   }
 
@@ -189,6 +201,28 @@ public abstract class Interpreter
     environ.define (sym, p);
   }
 
+  /** Enter a function into the current environment.
+   * The function is implemented using a static method.
+   * @param name the language-level name of the function.
+   * @param cname the fully-qualified name of the class containing the method.
+   * @param mname the name of the static method.
+   */
+  protected void define_method(String name, String cname, String mname)
+  {
+    environ.define(name, ClassMethods.apply(cname, mname));
+  }
+
+  protected void define_field (String name, String cname, String fname)
+  {
+    StaticFieldConstraint.define(environ, name, cname, fname);
+  }
+
+  protected void define_field (String name, String cname)
+  {
+    StaticFieldConstraint.define(environ, name, cname,
+				 Compilation.mangleNameIfNeeded(name));
+  }
+
   /** Enter a named function into the current environment. */
   public final void defineFunction(Named proc)
   {
@@ -238,8 +272,8 @@ public abstract class Interpreter
       name = ((Named) part).getName();
     else
       name = name.intern();
-    if (part instanceof Binding)
-      environ.addBinding((Binding) part);
+    if (part instanceof Symbol)
+      environ.addSymbol((Symbol) part);
     else
       environ.define(name, part);
   }
@@ -261,7 +295,7 @@ public abstract class Interpreter
       }
     catch (Exception ex)
       {
-	throw new WrappedException(ex);
+	throw new WrappedException("cannot load "+name, ex);
       }
   }
 
@@ -303,10 +337,12 @@ public abstract class Interpreter
 
   public abstract FormatToConsumer getFormat(boolean readable);
 
-  public Consumer getOutputConsumer(OutPort out)
+  public Consumer getOutputConsumer(Writer out)
   {
-    out.objectFormat = getFormat(false);
-    return out;
+    OutPort oport = out instanceof OutPort ? (OutPort) out
+      : new OutPort(out);
+    oport.objectFormat = getFormat(false);
+    return oport;
   }
 
   public Environment getNewEnvironment ()
@@ -316,12 +352,14 @@ public abstract class Interpreter
 
   public abstract String getName();
 
-  public abstract Lexer getLexer(InPort inp, gnu.text.SourceMessages messages);
+  public abstract Lexer getLexer(InPort inp, SourceMessages messages);
 
-  public abstract ModuleExp parse(Environment env, Lexer lexer)
+  public abstract Compilation parse(Environment env, Lexer lexer)
     throws java.io.IOException, gnu.text.SyntaxException;
 
-  public abstract ModuleExp parseFile (InPort port, gnu.text.SourceMessages messages);
+  public abstract Compilation parseFile (InPort port, boolean immediate,
+					 SourceMessages messages)
+    throws java.io.IOException, gnu.text.SyntaxException;
 
   public abstract Type getTypeFor(Class clas);
 
@@ -354,10 +392,12 @@ public abstract class Interpreter
       {
         if (spec instanceof Class)
           return getTypeFor((Class) spec);
-        if (spec instanceof String || spec instanceof Binding)
+        if (spec instanceof String)
           return getTypeFor(spec.toString());
+        if (spec instanceof Symbol)
+          return getTypeFor(((Symbol) spec).getName());
         if (spec instanceof CharSeq)
-          return gnu.bytecode.ClassType.make(spec.toString());
+          return ClassType.make(spec.toString());
       }
     return (Type) spec;
   }
@@ -378,10 +418,29 @@ public abstract class Interpreter
     else if (exp instanceof ReferenceExp)
       {
         ReferenceExp rexp = (ReferenceExp) exp;
-        Declaration decl = rexp.getBinding();
-        if (decl != null && ! decl.getFlag(Declaration.IS_UNKNOWN))
-          return getTypeFor(decl.getValue());
+        Declaration decl = Declaration.followAliases(rexp.getBinding());
         String name = rexp.getName();
+        if (decl != null)
+	  {
+	    name = decl.getName();
+	    exp = decl.getValue();
+	    if (decl.isAlias()
+		&& exp instanceof QuoteExp)
+	      {
+		Object val = ((QuoteExp) exp).getValue();
+		if (val instanceof Location)
+		  {
+		    Location loc = (Location) val;
+		    if (loc.isBound())
+		      return asType(loc.get());
+		    if (! (loc instanceof Named))
+		      return null;
+		    name = ((Named) loc).getName();
+		  }
+	      }
+	    else if (! decl.getFlag(Declaration.IS_UNKNOWN))
+	      return getTypeFor(exp);
+	  }
 	Object val = Environment.getCurrent().get(name);
 	if (val instanceof Type)
 	  return (Type) val;
@@ -397,6 +456,11 @@ public abstract class Interpreter
     return null;
   }
 
+  /** Used when defining a namespace alias (prefix), in the XML sense.
+   * Define in a namespace prefix NS is equivalent to defining a constant
+   * named NAMESPACE_PREFIX+"NS" whose value is the namespace URI. */
+  public static final String NAMESPACE_PREFIX = "$Namespace$";
+
   public static final int VALUE_NAMESPACE = 1<<0;
   public static final int FUNCTION_NAMESPACE = 1<<1;
 
@@ -444,10 +508,154 @@ public abstract class Interpreter
 
   public Procedure getPrompter()
   {
-    Binding pr = Environment.getCurrentBinding("default-prompter");
+    Symbol pr = Environment.getCurrentSymbol("default-prompter");
     return pr == null ? null : pr.getProcedure();
   }
 
+  /** Return the result of evaluating a string as a source expression. */
+  public final Object eval (String string) throws Throwable
+  {
+    return eval(new CharArrayInPort(string));
+  }
+
+  /** Evaluate expression(s) read from a Reader.
+   * This just calls eval(InPort).
+   */
+  public final Object eval (Reader in) throws Throwable
+  {
+    return eval(in instanceof InPort ? (InPort) in : new InPort(in));
+  }
+
+  /** Evaluate expression(s) read from an InPort. */
+  public final Object eval (InPort port) throws Throwable
+  {
+    CallContext ctx = CallContext.getInstance();
+    Consumer save = ctx.consumer;
+    try
+      {
+	ctx.consumer = ctx.vstack;
+	ctx.values = Values.noArgs;
+	eval(port, ctx);
+	return Values.make((gnu.lists.TreeList) ctx.vstack);
+      }
+    finally
+      {
+	ctx.vstack.clear();
+	ctx.consumer = save;
+      }
+  }
+
+  /** Evaluate a string and write the result value(s) on a Writer. */
+  public final void eval (String string, Writer out) throws Throwable
+  {
+    eval(new CharArrayInPort(string), out);
+  }
+
+  /** Evaluate a string and write the result value(s) to a PrintConsumer.
+   * This is to disambiguate calls using OutPort or XMLPrinter,
+   * which are both Writer and Consumer. */
+  public final void eval (String string, PrintConsumer out) throws Throwable
+  {
+    eval(string, getOutputConsumer(out));
+  }
+
+  /** Evaluate a string and write the result value(s) to a Consumer. */
+  public final void eval (String string, Consumer out) throws Throwable
+  {
+    eval(new CharArrayInPort(string), out);
+  }
+
+  /** Read expressions from a Reader and write the result to a Writer. */
+  public final void eval (Reader in, Writer out) throws Throwable
+  {
+    eval(in, getOutputConsumer(out));
+  }
+
+  /** Read expressions from a Reader and write the result to a Consumer. */
+  public void eval (Reader in, Consumer out) throws Throwable
+  {
+    InPort port = in instanceof Reader ? (InPort) in : new InPort(in);
+    CallContext ctx = CallContext.getInstance();
+    Consumer save = ctx.consumer;
+    try
+      {
+	ctx.consumer = out;
+	eval(port, ctx);
+      }
+    finally
+      {
+	ctx.consumer = save;
+      }
+  }
+
+  public void eval (InPort port, CallContext ctx) throws Throwable
+  {
+    SourceMessages messages = new SourceMessages();
+    Environment saveEnviron = Environment.getCurrent();
+    if (saveEnviron != environ)
+      Environment.setCurrent(environ);
+    Interpreter saveInterp = defaultInterpreter;
+    Interpreter.defaultInterpreter = this;
+    try
+      {
+	Compilation comp = parseFile(port, true, messages);
+	ModuleExp.evalModule(environ, ctx, comp);
+      }
+    finally
+      {
+	if (saveEnviron != environ && saveEnviron != null)
+	  Environment.setCurrent(saveEnviron);
+	Interpreter.defaultInterpreter = saveInterp;
+      }
+    if (messages.seenErrors())
+      throw new RuntimeException("invalid syntax in eval form:\n"
+				 + messages.toString(20));
+  }
+
+  public Object getDefaultSymbolValue (Symbol sym, boolean function)
+  {
+    String uri = sym.getNamespaceURI();
+    String name = sym.getName();
+    if (function || ! hasSeparateFunctionNamespace())
+      {
+	if (uri != null && uri.startsWith("class:"))
+	  return ClassMethods.apply(uri.substring(6), name);
+      }
+    // Last restort - check for unresolved namespace prefix.
+    int colon = name.indexOf(':');
+    Environment env = sym.getEnvironment();
+    if (colon > 1 & env != null)
+      {
+	String prefix = (NAMESPACE_PREFIX+name.substring(0, colon)).intern();
+	Object uri_val = env.get(prefix, null);
+	if (uri_val != null)
+	  {
+	    sym = Symbol.make(uri_val, name.substring(colon+1));
+	    return function ? getSymbolProcedure(sym)
+	      : getSymbolValue(sym);
+	  }
+      }
+    throw new UnboundSymbol(name);
+  }
+
+  public static Object getSymbolValue (Symbol sym)
+  {
+    Object val = sym.get(Symbol.UNBOUND);
+    if (val == Symbol.UNBOUND)
+      return getInterpreter().getDefaultSymbolValue(sym, false);
+    return val;
+  }
+
+  public static Procedure getSymbolProcedure (Symbol sym)
+  {
+    Object val = sym.getFunctionValue(Symbol.UNBOUND);
+    if (val == Symbol.UNBOUND)
+      val = sym.get(Symbol.UNBOUND);
+    if (val == Symbol.UNBOUND)
+      val = getInterpreter().getDefaultSymbolValue(sym, true);
+    return (Procedure) val;
+  }
+
   // The compiler finds registerEnvironment by using reflection.
   //
   // public static void registerEnvironment()
