@@ -27,9 +27,9 @@ public class InstanceOf extends Procedure2 implements CanInline, Inlineable
   static gnu.bytecode.ClassType typeType;
   static gnu.bytecode.Method instanceMethod;
 
-  public Expression inline (ApplyExp exp)
+  public Expression inline (ApplyExp exp, ExpWalker walker)
   {
-    return Invoke.inlineClassName(exp, 1, interpreter);
+    return Invoke.inlineClassName(exp, 1, (InlineCalls) walker);
   }
 
   public void compile (ApplyExp exp, Compilation comp, Target target)
