@@ -97,8 +97,7 @@ public class Lambda extends Syntax implements Printable
 	      }
 	    key_args = 0;
 	  }
-        else if (pair.car == "::" // && "::" is unbound FIXME
-                 && pair.cdr instanceof Pair)
+        else if (tr.matches(pair.car, "::") && pair.cdr instanceof Pair)
           pair = (Pair) pair.cdr;
 	else if (key_args >= 0)
 	  key_args++;
@@ -110,7 +109,7 @@ public class Lambda extends Syntax implements Printable
 	  lexp.min_args++;
 	bindings = pair.cdr;
       }
-    if (bindings instanceof String || bindings instanceof Binding)
+    if (bindings instanceof String || bindings instanceof Symbol)
       {
 	if (opt_args >= 0 || key_args >= 0 || rest_args >= 0)
 	  {
@@ -160,15 +159,20 @@ public class Lambda extends Syntax implements Printable
 	    mode = pair.car;
 	    continue;
 	  }
-	String name;
+	Object name;
 	Object defaultValue = defaultDefault;
 	Pair typeSpecPair = null;
         Pair p;
-	if (pair.car instanceof String || pair.car instanceof Binding)
+	if (tr.matches(pair.car, "::"))
+	  {
+	    tr.syntaxError("'::' must follow parameter name");
+	    return;
+	  }
+	if (pair.car instanceof String || pair.car instanceof Symbol)
           {
-            name = pair.car.toString();
+            name = pair.car;
             if (pair.cdr instanceof Pair
-                && (p = (Pair) pair.cdr).car == "::")
+                && tr.matches((p = (Pair) pair.cdr).car, "::"))
               {
                 if (! (pair.cdr instanceof Pair))
                   {
@@ -183,12 +187,12 @@ public class Lambda extends Syntax implements Printable
           }
 	else if (pair.car instanceof Pair
 		 && ((p = (Pair) pair.car).car instanceof String
-		     || p.car instanceof Binding)
+		     || p.car instanceof Symbol)
 		 && p.cdr instanceof Pair)
           {
-	    name = p.car.toString();
+	    name = p.car;
             p = (Pair) p.cdr;
-            if (p.car == "::")
+            if (tr.matches(p.car, "::"))
               {
                 if (! (p.cdr instanceof Pair))
                   {
@@ -235,7 +239,7 @@ public class Lambda extends Syntax implements Printable
                 if (p.cdr != LList.Empty)
                   {
                     tr.syntaxError("junk at end of specifier for parameter '"
-                                   + name + '\'');
+                                   + name + '\''+" after type "+p.car);
                     return;
                   }
               }
@@ -248,7 +252,9 @@ public class Lambda extends Syntax implements Printable
 	if (mode == optionalKeyword || mode == keyKeyword)
 	  lexp.defaultArgs[opt_args++] = tr.rewrite(defaultValue);
 	if (mode == keyKeyword)
-	  lexp.keywords[key_args++] = Keyword.make(name.toString());
+	  lexp.keywords[key_args++]
+	    = Keyword.make(name instanceof Symbol ? ((Symbol) name).getName()
+			   : name.toString());
 	Declaration decl = lexp.addDeclaration (name);
         if (bindings instanceof PairWithPosition)
           {
@@ -266,9 +272,9 @@ public class Lambda extends Syntax implements Printable
 	decl.noteValue(null);  // Does not have a known value.
 	tr.push(decl);
       }
-    if (bindings instanceof String || bindings instanceof Binding)
+    if (bindings instanceof String || bindings instanceof Symbol)
       {
-	Declaration decl = lexp.addDeclaration (bindings.toString());
+	Declaration decl = lexp.addDeclaration (bindings);
 	decl.setType(Compilation.scmListType);
 	decl.noteValue (null);  // Does not have a known value.
 	tr.push(decl);
@@ -281,8 +287,12 @@ public class Lambda extends Syntax implements Printable
   public void rewriteBody(LambdaExp lexp, Object body, Translator tr)
   {
     // Syntatic sugar:  <TYPE> BODY (or :: <TYPE> BODY) --> (as <TYPE> BODY)
-    if (body instanceof Pair && ((Pair) body).car == "::")
-      body = ((Pair) body).cdr;
+    boolean sawColons = false;
+    if (body instanceof Pair && tr.matches(((Pair) body).car, "::"))
+      {
+	body = ((Pair) body).cdr;
+	sawColons = true;
+      }
     if (body instanceof Pair && ((Pair) body).car == "<sequence>")
       {
         ClassType type = ClassType.make("gnu.lists.Consumer");
@@ -301,7 +311,8 @@ public class Lambda extends Syntax implements Printable
 	BeginExp bexp = (BeginExp) lexp.body;
 	Expression[] exps = bexp.getExpressions();
 	int len = exps.length;
-	if (len > 1)
+	// We allow ':: "TYPENAME" BODY' but not '"TYPENAME" BODY'
+	if (len > 1 && (sawColons || exps[0] instanceof ReferenceExp))
 	  {
 	    Expression rexp = exps[0];
 	    gnu.bytecode.Type rtype = tr.getInterpreter().getTypeFor(rexp);
@@ -318,6 +329,7 @@ public class Lambda extends Syntax implements Printable
                     value = new BeginExp(new_body);
                   }
                 lexp.body = Convert.makeCoercion(value, rexp);
+		lexp.body.setLine(bexp);
 		lexp.setReturnType(rtype);
 	      }
 	  }
