@@ -24,14 +24,15 @@
 (define (values #!rest (args :: <Object[]>))
   (invoke-static <gnu.mapping.Values> 'make args))
 
-(define (environment-bound? env sym)
-  (not (eq? ((primitive-virtual-method "gnu.mapping.Environment" "lookup"
-                                       "gnu.mapping.Binding"
-                                       (<symbol>))
-             env sym)
-       #!null)))
+(define (environment-bound? (env :: <gnu.mapping.Environment>)
+			    (sym :: <String>))
+  :: <boolean>
+  (invoke env 'isBound sym))
 
-(define (null-environment)
+;; The version number is not optiona according to R5RS.
+;; But since earlier versions of this implementation took 0 arguments,
+;; we'll make it optional for backwards compatibility, at least for now.
+(define (null-environment #!optional version)
   (static-field <kawa.standard.Scheme> 'nullEnvironment))
 
 (define (interaction-environment)
@@ -42,12 +43,16 @@
    (primitive-constructor <string> (<java.lang.String>))
    (constant-fold (primitive-static-method <kawa.Version> "getVersion"
 					   <java.lang.String> ()))))
-(define (procedure-property (proc :: <procedure>) key #!optional default)
-  (invoke proc 'getProperty key default))
-
 (define (set-procedure-property! proc :: <procedure> key value)
   (invoke proc 'setProperty key value))
 
+(define-procedure procedure-property
+  setter: set-procedure-property!
+  (begin
+    (define (procedure-property (proc :: <procedure>) key #!optional default)
+      (invoke proc 'getProperty key default))
+    procedure-property))
+
 ;;; The one-argument case is a standard DSSSL procedure.
 ;;; The multi-argument extension matches Guile.
 (define (error msg . args)
@@ -57,3 +62,10 @@
 		(call-with-output-string (lambda (port) (write arg port))))
 	      args))
   (apply throw 'misc-error msg args))
+
+(define (base-uri #!optional (node #!null))
+  (let ((uri (if (eq? node #!null)
+		 (invoke-static <gnu.kawa.functions.BaseUri> 'baseUri)
+		 (invoke-static <gnu.kawa.functions.BaseUri> 'baseUri node))))
+    (if (eq? uri #!void) #f uri)))
+
